@@ -1784,11 +1784,11 @@ suitable_encoding (const unsigned char *buffer, size_t length)
   if (*buffer & 0x80)
     return 0; /* If the MSB is set we assume that buffer represents a
                  negative number.  */
-  if (!*buffer)
-    return 0; /* Starting with a zero is pretty much a binary string.  */
 
   for (s=buffer; length; s++, length--)
     {
+      if (!*s)
+        return 0; /*binary*/
       if ( (*s < 0x20 || (*s >= 0x7f && *s <= 0xa0))
            && !strchr ("\b\t\v\n\f\r\"\'\\", *s))
         return 0; /*binary*/
